import pyfanuc
import subprocess
command = "pip3 install -e git+https://github.com/tonejca/pyfwlib.git@pyfanucable#egg=fwlib"
try:
    import fwlib
except ModuleNotFoundError:
    print("trying to execute:")
    print("\u0332".join(command))
    try:
        subprocess.call([command])
    except ModuleNotFoundError:
        print("failed, please first install fwlib by executing"+command)


import os
import platform
from setuptools import setup, Extension

fwlib_dir = os.path.join(os.getcwd(), "extern/fwlib")
libpath = os.path.join(fwlib_dir, "libfwlib32.so")
if not os.path.isfile(libpath):
    plat = "linux"
    machine = platform.machine()
    version = "1.0.5"
    if machine == "x86_64" or "AMD64":
        arch = "x64"
    elif machine == "i386":
        arch = "x86"
    elif machine.startswith("arm"):
        arch = "arm"
    else:
        pass
    fname = f"libfwlib32-{plat}-{arch}.so.{version}"
    # print(f"{fname=}", f"{libpath=}")
    os.symlink(fname, os.path.join(fwlib_dir, "libfwlib32.so"))
    os.symlink(fname, os.path.join(fwlib_dir, "libfwlib32.so.1"))

setup(
    name="fwlipy",
    version="0.0.2",
    description="",
    ext_modules=[
        Extension(
            "_fwlib",
            sources=["fwlib.i"],
            swig_opts=['-builtin'],
            #swig_opts=['-shadow'],
            # used during linking
            library_dirs=[fwlib_dir],
            include_dirs=[fwlib_dir],
            libraries=["fwlib32"],
            # used during runtime
            runtime_library_dirs=[fwlib_dir],
        )
    ],
    # used to locate c header files
    include_dirs=[fwlib_dir],
    package_data={
        "": [
            os.path.join(fwlib_dir, "libfwlib32-linux-x86.so.1.0.5"),
            os.path.join(fwlib_dir, "libfwlib32-linux-x64.so.1.0.5"),
            os.path.join(fwlib_dir, "libfwlib32-linux-armv7.so.1.0.5"),
            os.path.join(fwlib_dir, "fwlib32.h"),
        ]
    },
    install_requires=['swig>=4.0.2'],
)



import fwlib

from pprint import pprint


def main():
  ip = '127.0.0.1'
  port = 8193

  print('connecting to machine at {}:{}...'.format(ip, port))
  ret, libh = fwlib.cnc_allclibhndl3(ip, port, 10)

  if ret != 0:
      raise Exception('failed to connect')

  ret, cncids = fwlib.cnc_rdcncid(libh)

  machine_id = '-'.join([f'{v:08x}' for v in cncids])
  print(f'{machine_id=}')

  ret, sysinfo = fwlib.cnc_sysinfo(libh);
  pprint(sysinfo)

  ret, axes = fwlib.cnc_rdaxisname(libh);
  print(f'{axes=}')

  # class of data, kinds of data, num of axis
  # position data, [abs, mach, rel, dist to go]
  ret, axisdata = fwlib.cnc_rdaxisdata(libh, 1, [0, 1, 2, 3]);
  # servo data, [load, load current %, load current amps]
  ret, axisdata = fwlib.cnc_rdaxisdata(libh, 2, [0, 1, 2]);

  ret, (max_valid_fig, inprec, outprec) = fwlib.cnc_getfigure(libh, 0)

  ret, exeprgname = fwlib.cnc_exeprgname(libh)
  print(exeprgname)

  ret, exeprg_path = fwlib.cnc_exeprgname2(libh)
  print(f'{exeprg_path=}')

  ret, opmsg = fwlib.cnc_rdopmsg(libh)
  pprint(opmsg)

  ret, statinfo = fwlib.cnc_statinfo(libh)
  pprint(statinfo)

  fwlib.cnc_freelibhndl(libh)


if __name__ == '__main__':
    main()



# __init__.py

from importlib import resources

try:
    import tomllib
except ModuleNotFoundError:
    import tomli as tomllib

from .main import FocasException, UnableToReadMacro, UnableToReadAxis, FocasController

# Version of the realpython-reader package
__version__ = "0.0.2"



# -- Imports ------------------------------------------------------------------
import fwlib  # namespace
from types import TracebackType
from typing import Optional, Type
from time import sleep


class FocasException(Exception):
    pass


class UnableToReadMacro(FocasException):
    """Raised when a macro could not be read"""


class UnableToReadAxis(FocasException):
    """Raised when an axis could not be read"""


class FocasController:
    def __init__(self, ip: str, port: int = 8193, sample_rate: int = 10):
        self.ip = ip
        self.port = port
        self.sample_rate = sample_rate
        self.library_handle = None

    def __enter__(self):        #  -> Focas: gives an Error, so leave it!
        """Initialize the connector class"""

        self.connect()
        return self

    def __exit__(
        self,
        exception_type: Optional[Type[BaseException]],
        exception_value: Optional[BaseException],
        traceback: Optional[TracebackType],
    ) -> None:
        """Clean up the resources used by the storage class

        Parameters
        ----------

        exception_type:
            The type of the exception in case of an exception

        exception_value:
            The value of the exception in case of an exception

        traceback:
            The traceback in case of an exception

        """

        self.disconnect()

    def connect(self):
        ret=1
        n=0
        while ret != 0 and n<4:
            n=n+1
            ret, self.libh = fwlib.cnc_allclibhndl3(self.ip, self.port, self.sample_rate)
            if ret != 0:
                sleep(1/1000)
        if ret != 0:
            raise ConnectionError(
                f"Unable to connect to Focas (IP: {self.ip}, Port:"
                f" {self.port})"
            )

        self.library_handle = self.libh
        assert self.library_handle is not None, "Library handle not initialized"

    def disconnect(self):
        fwlib.cnc_freelibhndl(self.library_handle)

    def read_macro(self, register: int) -> int:
        
        ret, (_, dataFeature) = fwlib.cnc_rdmacror2(self.library_handle, [register])
        if ret != 0:
            raise UnableToReadMacro(f"Unable to read macro “{register}”")
        return dataFeature

    def read_axis(self, category: int, index: int) -> int:
        """
        example
        >>> url="192.168.1.10" 
        >>> with FocasController(url) as controller:
        ...     # 1...position, 0...absolute
        ...     axisdata = controller.read_axis(1,0)
        >>> len(axisdata)
        5
        >>> type(axisdata[0])
        <class 'axdata'>
        
        """
        ret, axisdata = fwlib.cnc_rdaxisdata(self.library_handle, category, [index])
        if ret != 0:
            raise UnableToReadAxis(f"Unable to read axis data “{category}”, “{index}”")
        return axisdata


if __name__ == "__main__":
    # nc_data = []
    tuple = [[1,2,3,5], [0,1]]
    focas_controller = FocasController("192.168.1.10")
    with focas_controller:
        try:
            # axis_tuples = [[1, 0], [2, 0], [3, 0], [5, 0], [5, 1]]
            # for tuple in axis_tuples:
            axisdata = focas_controller.read_axis(tuple[0], tuple[1])
            # for axis in axisdata:
            #     nc_data.append(axis[1])
            # print(nc_data)
        except (ConnectionError, UnableToReadMacro, UnableToReadAxis) as error:
            print(error)
            focas_controller.disconnect()
    print(axisdata)
